//+------------------------------------------------------------------+
//|                                                  Dynamic RSI.mq4 |
//|          https://www.mql5.com/ru/users/romankiverin/publications |
//|                                         https://t.me/RomanKiverin|
//+------------------------------------------------------------------+
#property version     "1.0"
#property copyright   "Copyright 2023, Roman Kiverin"
#property link        "https://www.mql5.com/ru/users/romankiverin/publications"
#property strict

#property description "Dynamic RSI\nhttps://t.me/RomanKiverin"
#property indicator_separate_window

#property indicator_buffers 14

// --- settings
extern int     Maxbar   = 3000;

input double   DZbuy       =  0.1;          // Buyers Dynamic Zone Probability
input double   DZsell      =  0.1;          // Sellers Dynamic Zone Probability
input int      PeriodRSI   =  14;           // Period RSI
input int      LookBack    =  80;           // Look Back PeriodRSI
input int      LevUpRSI    =  70;           // Level Up for RSI
input int      LevDnRSI    =  30;           // Level Down for RSI
input int      LevUpBand   =  55;           // Level Up for Median BBands
input int      LevDnBand   =  45;           // Level Down for Median BBands
input color    RSIColor    =  clrIndigo;    // RSI Line Color
input color    MaxColor    =  clrFireBrick;    // Sell Signall Color
input color    MinColor    =  clrLime;         // Buy Signall Color
input color    BBandsUpCol =  clrMaroon;       // BBands Up Line Color
input color    BBandsLoCol =  clrDarkGreen;    //BBands Lo Line Color
input color    MidBandColor=  clrDarkGoldenrod;//BBands Mid Line Color  



//--- indicator buffers
double RSILineBuffer[];
double MaxLineBuffer[];
double MinLineBuffer[];
double BandsMiBuf[];
double BandsUpBuf[];
double BandsLoBuf[];
double DnLineBuffer[];
double WMALineBuffer[];
double UpLineBuffer[];
double RSIValueBuffer[];
double DeltaBuffer[];
double UpBuffer[];
double DnBuffer[];
double LineBuffer[];

//double UpSigBuf[];
//double DnSigBuf[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,RSILineBuffer);
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1,RSIColor);
   SetIndexEmptyValue(0,0);
      
   SetIndexBuffer(1,MaxLineBuffer);
   SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,2,MaxColor);
   SetIndexEmptyValue(1,0);

   SetIndexBuffer(2,MinLineBuffer);
   SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,2,MinColor);
   SetIndexEmptyValue(2,0);
   
   SetIndexBuffer(3,BandsMiBuf);
   SetIndexStyle(3,DRAW_LINE,STYLE_SOLID,1,MidBandColor);
   SetIndexEmptyValue(3,0);
   
   SetIndexBuffer(4,BandsUpBuf);
   SetIndexStyle(4,DRAW_LINE,STYLE_SOLID,1,BBandsUpCol);
   SetIndexEmptyValue(4,0);
   
   SetIndexBuffer(5,BandsLoBuf);
   SetIndexStyle(5,DRAW_LINE,STYLE_SOLID,1,BBandsLoCol);
   SetIndexEmptyValue(5,0);
   
   SetIndexBuffer(6,DnLineBuffer);
   SetIndexStyle(6,DRAW_NONE);

   SetIndexBuffer(7,WMALineBuffer);
   SetIndexStyle(7,DRAW_NONE);

   SetIndexBuffer(8,UpLineBuffer);
   SetIndexStyle(8,DRAW_NONE);

   SetIndexBuffer(9,RSIValueBuffer);
   SetIndexStyle(9,DRAW_NONE);

   SetIndexBuffer(10,DeltaBuffer);
   SetIndexStyle(10,DRAW_NONE);

   SetIndexBuffer(11,UpBuffer);
   SetIndexStyle(11,DRAW_NONE);

   SetIndexBuffer(12,DnBuffer);
   SetIndexStyle(12,DRAW_NONE);

   SetIndexBuffer(13,LineBuffer);
   SetIndexStyle(13,DRAW_NONE);
   
   /*SetIndexBuffer(14,UpSigBuf);
   SetIndexStyle(14,DRAW_HISTOGRAM,STYLE_SOLID,5,MaxColor);
   SetIndexEmptyValue(14,0);
   
   SetIndexBuffer(15,DnSigBuf);
   SetIndexStyle(15,DRAW_HISTOGRAM,STYLE_SOLID,5,MinColor);
   SetIndexEmptyValue(15,0);*/
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
     if (Maxbar > Bars) Maxbar = Bars;
   for(int i = MathMin(rates_total - prev_calculated, Maxbar - LookBack - 5); i >= 0; i --)
     {
      RSIValueBuffer[i] = iRSI(NULL, 0, PeriodRSI, PRICE_CLOSE, i);

      UpBuffer[i]=RSIValueBuffer[ArrayMaximum(RSIValueBuffer,LookBack,i)];
      DnBuffer[i]=RSIValueBuffer[ArrayMinimum(RSIValueBuffer,LookBack,i)];
      DeltaBuffer[i]=(UpBuffer[i]-DnBuffer[i]) / 2.0;
      WMALineBuffer[i]=iMAOnArray(DeltaBuffer,0,PeriodRSI,0,MODE_LWMA,i) + iMAOnArray(DnBuffer,0,PeriodRSI,0,MODE_LWMA,i);
      UpLineBuffer[i]=UpBuffer[i] - WMALineBuffer[i] * DZbuy;
      DnLineBuffer[i]=DnBuffer[i] + WMALineBuffer[i] * DZsell;
      LineBuffer[i]=(4 * RSIValueBuffer[i] + 3 * RSIValueBuffer[i+1] + 2 * RSIValueBuffer[i+2] + RSIValueBuffer[i+3]) / 10;
        {
         if(LineBuffer[i] > UpLineBuffer[i])
          {
            MaxLineBuffer[i+1] = LineBuffer[i+1];
            RSILineBuffer[i + 1] = MaxLineBuffer[i + 1] = LineBuffer[i + 1];
            MinLineBuffer[i + 1] = RSILineBuffer[i] =  MinLineBuffer[i] = 0;
          }
         else
         if(LineBuffer[i] < DnLineBuffer[i])
           {
            MinLineBuffer[i+1] = LineBuffer[i+1];
            RSILineBuffer[i + 1] = MinLineBuffer[i + 1] = LineBuffer[i + 1];
            MaxLineBuffer[i + 1] = MaxLineBuffer[i] = RSILineBuffer[i] = 0;
           }
         else
           {
            RSILineBuffer[i] = LineBuffer[i];
            RSILineBuffer[i + 1] = LineBuffer[i + 1];
           }
        }
       BandsMiBuf[i]=iBandsOnArray(RSILineBuffer,0,PeriodRSI,2,0,MODE_MAIN,i+1);    
       BandsUpBuf[i]=iBandsOnArray(RSILineBuffer,0,PeriodRSI,2,0,MODE_UPPER,i+1);
       BandsLoBuf[i]=iBandsOnArray(RSILineBuffer,0,PeriodRSI,2,0,MODE_LOWER,i+1);
               
        /*if(RSILineBuffer[i+1]>LevUpRSI) UpSigBuf[i+1]=(LevUpRSI-(LevUpRSI-1))+(RSILineBuffer[i+1]-LevUpRSI);
        else UpSigBuf[i+1]=LevUpRSI-(LevUpRSI-1);
        if(RSILineBuffer[i+1]<LevDnRSI) DnSigBuf[i+1]=(RSILineBuffer[i+1]-(RSILineBuffer[i+1]-1))+(LevDnRSI-RSILineBuffer[i+1]);
        else DnSigBuf[i+1]=0;*/
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
